unit Terrain;

{unit qui contient une fiche dont le TImage est un terrain pour
diverses manips}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls,

  Maths, Utils_Dialogs;

type
  TTerrainForm = class(TForm)
    TheTerrain: TImage;
    Button1: TButton;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }

    FXMaxWanted, FYMaxWanted, FXMinWanted, FYMinWanted,
    FXRange, FYRange : Single;
    FOrientationRepere : TOrientationRepere ;
    FYOSigne : TSigne ;
    FOriginePixel : TPoint;
    FCanCalcEchelles : Boolean ;
    FXMaxPixels, FYMaxPixels, FMargeX, FMargeY : integer ;

    EchelleX : Single ; {en cm/pixel}
    EchelleY : Single ; {en cm/pixel}

    procedure SetXMaxWanted (const xmw : Single);
    procedure SetYMaxWanted (const ymw : Single);
    procedure SetXMinWanted (const xmw : Single);
    procedure SetYMinWanted (const ymw : Single);
    procedure SetOrientationRepere (const nor : TOrientationRepere);
    procedure SetYOSigne (const nyos : TSigne);
    procedure SetOriginePixel (const nop : TPoint);
    procedure SetMargeX (const nx : integer);
    procedure SetMargeY (const ny : integer);
    procedure Setbrush (const nb : TBrush);
    procedure SetPen   (const np : TPen);
    function  ReadPen : TPen;
    function  ReadBrush : TBrush;

    procedure CalculeEchelles ;

  public
    { Public declarations }

    { la taille du cadre de travail en pixels }
    {pas de write, l'utilisateur n'y touche pas }
    property XMaxPixels : integer
             read FXMaxPixels ;
    property YMaxPixels : integer
             read FYMaxPixels ;

    { les tailles dbjet, en rel, que manipulent les
    simulations}
    property XMaxWanted : Single
             read FXMaxWanted write SetXMaxWanted  ;
    property YMaxWanted : Single
             read FYMaxWanted write SetYMaxWanted   ;
    property XMinWanted : Single
             read FXMinWanted write SetXMinWanted  ;
    property YMinWanted : Single
             read FYMinWanted write SetYMinWanted   ;
    { donnent taille cadre rel , pas modifiables
    (passer par XMinWanted & co) }
    property XRange : Single
             read FXRange ;
    property YRange : Single
             read FYRange ;

    {L'Orientation }
    property OrientationRepere : TOrientationRepere
             read FOrientationRepere write SetOrientationRepere;
    property YOSigne : TSigne
             read FYOSigne write SetYOSigne ;

    { la marge }
    property MargeX  : integer
             read FMargeX write SetMargeX ;
    property MargeY  : integer
             read FMargeY write SetMargeY ;

    {l'origine dans le repre : automatique !!}
    property OriginePixel : TPoint    { origine en v.pixels}
             read FOriginePixel write SetOriginePixel       ;
    {permssion de calculer l'chelle, histoire d'viter
    des calculs en srie quand on change plusieurs trucs
    ensemble }
    property CanCalcEchelles : Boolean
             read FCanCalcEchelles write FCanCalcEchelles ;

    { la brosse et le Pen }
    property Brush : TBrush
             read ReadBrush write SetBrush ;
    property Pen  : TPen
             read ReadPen write SetPen ;

{ RAPPEL : toute modif du cadre ne fait pas bouger l'image;
une image 400x400 rduite, streche ou pas, peut se
laisser dessiner toujours en 400x400 }

    { fonctions que peuvent utiliser les utilisateurs }

    {deux fonctions qui font la conversion coord relles
    vers pixels }
    function  ttx (x : Single) : integer;
    function  tty (y : Single) : integer ;
    { donnent la taille en pixels de dx ou dy }
    function  XPxDist (dx : Single) : integer;
    function  YPxDist (dy : Single) : integer;

    procedure HideTerrain;
    procedure ShowTerrain ; { pour viter effets disgracieux }
    {procedure TerrainSize (const sx, sy : integer);
    { change la taille du dessin (et de la fiche)}

    procedure teInit (const cl : TColor; x, y : integer);
    procedure teClear (const cl : TColor);
    procedure teReellesCoords (const xmin,xmax,ymin,ymax : Single);

    procedure teMoveTo (const x,y : Single);
    procedure teLineTo (const x,y : Single);

    procedure teCarre  (const x,y,c : Single); { carr }
    procedure teCarrePlein(const x,y,c : Single);
    procedure teCase    (const x,y,c : Single); { case (ie 1 pixel en -}
    procedure teCasePleine (const x,y,c : Single);
    { cases dans un rseau de nbr cases de long (pas oubler si commence  0!)}
    procedure teCaseR    (const x,y : Single; nbr : integer);
    procedure teCasePleineR (const x,y : Single; nbr : integer);

    procedure teDessineMire ; {dmo }
    procedure teDessineMire2 ; {dmo }


  end;

var
  TerrainForm: TTerrainForm;


{**********************************************************}
implementation

{$R *.DFM}

function TTerrainForm.ttx (x : Single) : integer;
begin
  Result := Trunc((x-XMinWanted)/EchelleX) ;
end ;

function TTerrainForm.XPxDist (dx : Single) : integer;
begin
  Result := Abs(Trunc(dx/EchelleX)) ;
end ;

function  TTerrainForm.tty (y : Single) : integer ;
var bidon : SmallInt ;
begin
   bidon := Trunc((y-YMinWanted)/EchelleY) ;
  { est ngatif si on est en cartsien ; on doit ajouter
  la base}
  if OrientationRepere = orCartesien then
    bidon := YMaxPixels + bidon ;

  Result := bidon ;
end;

function  TTerrainForm.YPxDist (dy : Single) : integer;
begin
  Result := Abs(Trunc(dy/EchelleY)) ;
end ;

procedure TTerrainForm.FormCreate(Sender: TObject);
begin
  HideTerrain;
  CanCalcEchelles := False ;

  OrientationRepere := orCartesien ;

  {valeurs par dfaut }

  { les marges }
  MargeX := 20 ;
  MargeY := 20;

  {la taille du form, qui redfinit auto la taille du Tmage}
  Height:= 600;
  Width := 600;

  { les coordonnes rlles }
  teReellesCoords (-1,1,-1,1);

  OrientationRepere := orCartesien;

  teInit (clBlack, TheTerrain.Width, TheTerrain. Height);

  FXMaxPixels := TheTerrain.Picture.Width;
  FYMaxPixels := TheTerrain.Picture.Height;

  {--------------------------------------- fin init}
  CanCalcEchelles := True ;
  CalculeEchelles ;
  ShowTerrain ;

end;

procedure TTerrainForm.CalculeEchelles ;
  begin
    if CanCalcEchelles and (XMaxPixels*YMaxPixels > 0)then begin
      EchelleX := XRange/XMaxPixels ;   {cm/pixel}
      EchelleY := YRange/YMaxPixels ;
      {le signe dfini par OrientationEchelle }
      EchelleY := EchelleY*YOSigne ;
    end ;
  end ;

procedure TTerrainForm.SetXMaxWanted (const xmw : Single);
begin
    FXMaxWanted := xmw;
    FXRange := FXMaxWanted-FXMinWanted ;
    CalculeEchelles ;
  end ;

procedure TTerrainForm.SetYMaxWanted (const ymw : Single);
  begin
    FYMaxWanted := ymw;
    FYRange := FYMaxWanted-FYMinWanted ;
    CalculeEchelles ;
  end ;

procedure TTerrainForm.SetXMinWanted (const xmw : Single);
begin
    FXMinWanted := xmw;
    FXRange := FXMaxWanted-FXMinWanted ;
    CalculeEchelles ;
  end ;

procedure TTerrainForm.SetYMinWanted (const ymw : Single);
  begin
    FYMinWanted := ymw;
    FYRange := FYMaxWanted-FYMinWanted ;
    CalculeEchelles ;
  end ;

procedure TTerrainForm.SetOrientationRepere (const nor : TOrientationRepere);
{ si on passe en cartsien , les axes s'orentent vers le haut}
{on ne touche pas  l'origine, c  part }
  begin
      FOrientationRepere := nor ;
      if nor = orCartesien then FYOSigne := -1
                           else FYOSigne := +1 ;
      CalculeEchelles ;
  end ;

procedure TTerrainForm.SetYOSigne (const nyos : TSigne);
{ passe ventuellement en cartsien
  NB : FYOSigne est modifi DIRECTEMENT par SetOrientationRepere)}
  begin
    if nyos=1 then OrientationRepere:=orEcran
    else if nyos=-1 then OrientationRepere:=orCartesien
         else StopBox ('Dfinition de YOSigne nul dans SetYOSigne !!');
    CalculeEchelles ;
  end ;

procedure TTerrainForm.SetOriginePixel (const nop : TPoint);
begin
  FOriginePixel := nop;
  CalculeEchelles;
  {prvoir redessin + tard  ?? }
end;

procedure TTerrainForm.SetMargeX (const nx : integer);
begin
  TheTerrain.Left := nx;
  FMargeX := nx ;
end;

procedure TTerrainForm.SetMargeY (const nY : integer);
begin
  TheTerrain.Top := ny;
  FMargeY := ny ;
end;

procedure TTerrainForm.SetBrush;
  begin
    TheTerrain.Canvas.Brush := nb
  end ;

procedure TTerrainForm.SetPen;
  begin
    TheTerrain.Canvas.Pen := np
  end ;

function TTErrainForm.ReadBrush : TBrush ;
  begin
    Result := TheTerrain.Canvas.Brush
  end ;

function TTErrainForm.ReadPen : TPen ;
  begin
    Result := TheTerrain.Canvas.Pen
  end ;

procedure TTerrainForm.FormResize(Sender: TObject);
var POuvaitDejaCalcEch : Boolean ;
begin
  {Resize du terrain : le TImage doit s'adapter }
  PouvaitDejaCalcEch:=CanCalcEchelles;
  CanCalcEchelles :=False;
  TheTerrain.Top  := MargeY;
  TheTerrain.Left := MargeX;

  TheTerrain.Height := Max(1,Self.ClientHeight-2*MargeY);
  TheTerrain.Width  := Max(1,Self.ClientWidth-2*MargeX);
  FXMaxPixels := TheTerrain.Picture.Width;
  FYMaxPixels := TheTerrain.Picture.Height;

  { seulement si on n'tait pas en pleine init ou redfinition}
  if PouvaitDejaCalcEch then begin

    CanCalcEchelles:=True;
    CalculeEchelles ;{recalcul pour changer l'chelle }

  {calcul du nouveau pixel origine }
    OriginePixel := Point (ttx(0),tty(0));

  {  TheTerrain.Refresh ;
  { le reste tent aux proprits de Stretch &co) }
  end ;

end;

procedure TTerrainForm.HideTerrain ;
begin
  TheTerrain.Visible :=  False ;
end ;
procedure TTerrainForm.ShowTerrain ;
begin
  TheTerrain.Visible :=  True ;
end ;


{procedure TTerrainForm.TerrainSize (const sx, sy : integer);
{ pas garantie...}
 { begin
    HideTerrain ;
    Self.Height := sy+2*MargeY;
    Self.Width  := sx+2*MargeX;
     { et FormResize est appel }
 {   ShowTerrain;
  end ;
  }
procedure TTerrainForm.teInit (const cl : TColor; x, y : integer);
  begin
    TheTerrain.Canvas.Brush.Color:=cl;
    TheTerrain.Canvas.Pen.Color:=cl;
    TheTerrain.Canvas.Rectangle (0,0,X, Y);
  end ;

procedure TTerrainForm.teReellesCoords (const Xmin,Xmax,Ymin,Ymax : Single);
  var PouvaitCalcEchelles : Boolean;
  begin
    PouvaitCalcEchelles := CanCalcEchelles;
    CanCalcEchelles := False ;
    XminWanted := xmin;
    XmaxWanted := xmax;
    YminWanted := ymin;
    YMaxWanted := ymax;
    CanCalcEchelles :=  PouvaitCalcEchelles;
    CalculeEchelles;
  end ;

procedure TTerrainForm.teClear (const cl : TColor);
  begin
    TheTerrain.Canvas.Brush.Color:=cl;
    TheTerrain.Canvas.Rectangle (0,0,XmaxPixels, YMaxPixels);
  end ;

procedure TTerrainForm.teMoveTo (const x,y : Single);
begin
  TheTerrain.Canvas.MoveTo(ttx(x),tty(y));
end ;

procedure TTerrainForm.teLineTo (const x,y : Single);
begin
  TheTerrain.Canvas.LineTo(ttx(x),tty(y));
end;

procedure TTerrainForm.teCarre (const x,y,c : Single);
{ c : ct}
  var xx, xxc,yy, yyc : integer ;
  begin
    xx:=ttx(x);
    yy:=tty(y);
    xxc:=ttx(x+c);
    yyc:=tty(y+c);
    with TheTerrain.Canvas do begin
      MoveTo(xx, yy);
      LineTo(xxc, yy);
      LineTo(xxc, yyc);
      LineTo(xx, yyc);
      LineTo(xx, yy);
    end ;
  end ;

procedure TTerrainForm.teCarrePlein (const x,y,c : Single);
  begin
    TheTerrain.Canvas.rectangle (ttx(x), tty(y), ttx(x+c), tty(y+c));
  end ;

procedure TTerrainForm.teCase (const x,y,c : Single);
{ c : ct}
  var xx, xxc,yy, yyc : integer ;
  begin
    xx:=ttx(x);
    yy:=tty(y);
    xxc:=ttx(x+c)-1;
    yyc:=tty(y+c)-1;
    with TheTerrain.Canvas do begin
      MoveTo(xx, yy);
      LineTo(xxc, yy);
      LineTo(xxc, yyc);
      LineTo(xx, yyc);
      LineTo(xx, yy);
    end ;
  end ;

procedure TTerrainForm.teCasePleine(const x,y,c : Single);
var xx, yy, cx, cy : integer ;
  begin
    xx:=ttx(x);
    yy:=tty(y);
    cx := ttx(x+c)-1 ;
    cy := tty(y+c)-1;
    TheTerrain.Canvas.Rectangle (xx,yy,cx,cy);
  end ;

procedure TTerrainForm.teCaseR (const x,y : Single; nbr : integer);
{ c : ct}
  var xx, xxc,yy, yyc : integer ;
  begin
    xx:=ttx(x);
    yy:=tty(y);
    xxc:=ttx(x+XRange/nbr)-1;
    yyc:=tty(y+YRange/nbr)-1;
    with TheTerrain.Canvas do begin
      MoveTo(xx, yy);
      LineTo(xxc, yy);
      LineTo(xxc, yyc);
      LineTo(xx, yyc);
      LineTo(xx, yy);
    end ;
  end ;

procedure TTerrainForm.teCasePleineR (const x,y : Single; nbr : integer);
  begin
    TheTerrain.Canvas.rectangle
              (ttx(x), tty(y), ttx(x+XRange/nbr)-1, tty(y+YRange/nbr)-1);
  end ;


procedure TTerrainForm.teDessineMire ;
  begin
    {redimensionne }
  {  TerrainSize (100,100);

    {on reste en cartsien }

    with TheTerrain.Canvas do begin
      Pen.Color := clGreen;
      Pen.Width := 1;
      teMoveTo(-1,-1);
      teLineTo (1,1);
      Pen.Color := clRed;
      Pen.Width := 1;
      teMoveTo(-1,1);
      teLineTo(1,-1);

      Pen.Color := clBlue;
      Pen.Width := 1;

      MoveTo (0,50); Lineto(50,50);
    end ;

  end ;

procedure TTerrainForm.teDessineMire2 ;
  begin
    {redimensionne }

    {on passe en cran }
    teReellesCoords (-0.2,0.2,-0.2,0.2);

    OrientationRepere := orEcran ;

    with TheTerrain.Canvas do begin
      Pen.Color := clWhite;
      Pen.Width := 3;
      teMoveTo(0,0);
      teLineTo (0.1,0.1);
      Pen.Color := clBlue;
      Pen.Width := 3;
      teMoveTo(0.1,0.1);
      teLineTo(0.1,-0.1);

      Pen.Color := clBlue;
      Pen.Width := 1;

      teCarre (0,0, 0.1);
      teCarre (0,0, 0.05);
      teCarre (0,0, 0.03);

      MoveTo (0,50); Lineto(50,50);
    end ;

  end ;

procedure TTerrainForm.Button1Click(Sender: TObject);
begin
  teDessineMire ;
end;

procedure TTerrainForm.Button2Click(Sender: TObject);
begin
  teDessineMire2 ;
end;

end.
